# frozen_string_literal: true

require 'open3'

module DependencyUpdater
  class GemHandler

    # captures Using|Installing, gem name, new version, old version
    UPDATE_REGEX = /(Using|Installing)\s(.+?)\s(\d+?\.\d+?\.\d+?)\s.was\s(\d+?\.\d+?\.\d+?)/

    def initialize(git_object)
      @gitlab    = git_object
      @gems      = []
      @updated_capistrano = nil
    end

    def update_gems
      cmd = 'bundle update'
      output, err, status = Open3.capture3 cmd

      @gems = updated_gems output
      return false unless @gems.present?

      actions = actions_from_filepath('Gemfile.lock')
      @gitlab.commit("Update gems", actions)
      return true
    end

    def update_capistrano_deploy
      return unless check_for_capistrano

      actions = actions_from_filepath('config/deploy.rb')
      @gitlab.commit('Update deploy.rb with new Capistrano version', actions)
    end

    private

    def updated_gems output
      output.scan(UPDATE_REGEX).map do |match|
        {
          name: match[1],
          old_version: match[3],
          new_version: match[2]
        }
      end
    end

    def actions_from_filepath(filepath)
      actions = []
      content = case filepath
                when 'config/deploy.rb' then new_deploy_file_text(filepath)
                when 'Gemfile.lock'     then File.read(filepath).strip
                end

      actions << @gitlab.build_commit_action(filepath, content)
    end

    def check_for_capistrano
      @updated_capistrano = @gems.select { |gem| gem[:name] == 'capistrano' }.first
    end

    def new_deploy_file_text(filepath)
      File.read(filepath).strip.gsub(/lock '\d+?\.\d+?\.\d+?'/, "lock '#{@updated_capistrano[:new_version]}'")
    end
  end
end
