# frozen_string_literal: true

require 'gitlab'
require 'dependency_updater/messengers'

module DependencyUpdater
  class GitlabHandler
    attr_accessor :gitlab
    include Messengers

    def initialize
      config             = DependencyUpdater.configuration
      @project_id        = config.project_id
      @production_branch = config.production_branch
      @target_branch     = config.merge_target_branch
      @new_branch        = nil
      @mr                = nil
      @messenger = Messengers::GitlabMessenger.new
      @gitlab    = Gitlab.client
      @today     = Time.zone.today
    end

    def new_branch
      @new_branch = unique_branch_name
      @gitlab.create_branch(@project_id, @new_branch, @production_branch)
      @messenger.new_branch(@new_branch)
    end

    def build_commit_action(filepath, content)
      {
        action: 'update',
        file_path: filepath,
        content: content
      }
    end

    def commit(message, actions)
      signed_msg = "#{message} -- using Dependency Updater'"
      @gitlab.create_commit(@project_id, @new_branch, signed_msg, actions)
      @messenger.commit(signed_msg)
    end

    def create_mr
      title = "WIP: Dependency Update, #{@today}"
      @mr = @gitlab.create_merge_request(@project_id, title, {
        source_branch: @new_branch,
        target_branch: @target_branch,
        description: 'Dependency Updater has updated the gems and/or ruby version declarations.',
        remove_source_branch: true
      })
      @messenger.new_mr(@mr.title, @mr.web_url)
    end

    def backout
      @gitlab.delete_branch(@project_id, @new_branch)
      @messenger.backing_out(@new_branch)
    end

    private

    def unique_branch_name
      updater_branches = @gitlab.branches(@project_id, { search: '^dependency-update' }).map(&:name)
      highest_id = updater_branches.map { |b| b.slice(/\/update-(\d+)/, 1).to_i }.max
      highest_id = 0 if highest_id.nil?
      "dependency-update/#{@today}/update-#{highest_id + 1}"
    end
  end
end
