# frozen_string_literal: true

require 'rest-client'

module DependencyUpdater
  class RubyHandler

    SOURCE = 'https://www.ruby-lang.org/en/downloads/releases/'
    FILES  = [
      '.ruby-version',
      '.htaccess',
      'lib/capistrano/templates/htaccess.erb'
    ]

    def initialize(git_object)
      @gitlab = git_object
      @original_version     = ''
      @latest_patch_version = ''
    end

    def find_version_data
      find_original_version
      find_latest_patch_version
    end
    
    def update_project_files
      return false if up_to_date

      actions = []
      FILES.each do |filepath|
        next unless File.exist? filepath

        new_text = File.read(filepath).strip.gsub(@original_version, @latest_patch_version)
        actions << @gitlab.build_commit_action(filepath, new_text)
      end
      message = "Update Ruby version declarations to #{@latest_patch_version}"
      @gitlab.commit(message, actions)
      return true
    end

    private

    def find_original_version
      @original_version = File.read('.ruby-version').strip
    end

    def find_latest_patch_version
      minor_used = @original_version.slice(/(\d+?\.\d+?)\.\d+?/, 1)
      minor_available = RestClient.get(SOURCE).body.scan(/Ruby\s(#{minor_used}\.\d+?)[^-]/).flatten
      highest_patch_available = minor_available.map { |v| v.slice(/\d+?$/).to_i }.max
      @latest_patch_version = "#{minor_used}.#{highest_patch_available}"
    end

    def up_to_date
      @original_version == @latest_patch_version
    end
  end
end
